@@ -1,6 +1,6 @@
 <?php
 /**
- * Copyright 2007-2016 Horde LLC (http://www.horde.org/)
+ * Copyright 2007-2017 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
@@ -100,14 +100,20 @@ class Horde_Rpc_Jsonrpc extends Horde_Rpc
     function _raiseError($error, $request)
     {
         $code = $userinfo = null;
-        if (is_a($error, 'PEAR_Error')) {
+        if ($error instanceof Exception) {
+            if ($error instanceof Horde_Exception) {
+                $userinfo = $error->details;
+            }
+            $code = $error->getCode();
+            $error = $error->getMessage();
+        } elseif ($error instanceof PEAR_Error) {
             $code = $error->getCode();
             $userinfo = $error->getUserInfo();
             $error = $error->getMessage();
         }
         $error = array('name' => 'JSONRPCError',
                        'code' => $code ? $code : 999,
-                       'message' => $error);
+                       'message' => (string)$error);
         if ($userinfo) {
             $error['error'] = $userinfo;
         }
